import {readFile} from 'node:fs/promises';
import {env} from 'node:process';
import {WASI} from 'node:wasi';

console.info("\t(Node.js host) Version: " + process.version);
console.info("\t(Node.js host) Current directory: " + process.cwd());
console.info("\t(Node.js host) Barcode: " + process.argv[2]);
const module = await WebAssembly.compile(await readFile(new URL('../build/Barcode_WASI.wasm', import.meta.url)));

// https://nodejs.org/api/wasi.html#new-wasioptions
const wasi = new WASI({
    args: process.argv, // Node.js command line arguments are passed to the WebAssembly module
    env, // Caution here: 'process.env' is *ACTUALLY* passed to 'wasi'...
    preopens: {
        '/sandbox': process.cwd() // Access to resource(s) in current directory...
    },
    returnOnExit: true, // 'wasi.start()' returns the exit code to Node.js...
    version: 'preview1' // Mandatory!
});
// Imported object is required, i.e., the API itself to access resource(s):
const instance = await WebAssembly.instantiate(module, wasi.getImportObject());
// Node.js WASI API imposes that the start method must be explicitly named '_start':
const checksum = wasi.start(instance); // '0', '-1' or '-2'
console.info("\t(Node.js host) Checksum: " + checksum);
process.exit(checksum); // Node.js returns the exit code to Operating System -OS- (Windows: 'Write-Output $?' -> 'True' or 'False')...

